#include <iostream>
#include <string>
#include <iomanip>
#include <SerialStream.h>

using namespace LibSerial;
 
class K_serial{
	public:
	K_serial(){
		//default constructor;
		char* DEVICE = "/dev/ttyS0";
		SERIAL_PORT_DEVICE = DEVICE;
		std::cout<<"K_serial::Default constructor. Setting " << SERIAL_PORT_DEVICE <<std::endl;
		serial_port.Open( SERIAL_PORT_DEVICE ) ;
		if ( ! serial_port.good() ) 
		{
			std::cerr << "Error: Could not open serial port " 
				<< SERIAL_PORT_DEVICE 
				<< std::endl ;
			exit(1) ;
		}
		//
		// Set the baud rate of the serial port.
		//
		serial_port.SetBaudRate( SerialStreamBuf::BAUD_2400) ;
		if ( ! serial_port.good() ) 
		{
			std::cerr << "Error: Could not set the baud rate." << std::endl ;
			exit(1) ;
		}
		//
    		// Set the number of data bits.
		//
		serial_port.SetCharSize( SerialStreamBuf::CHAR_SIZE_8 ) ;
		if ( ! serial_port.good() ) 
		{
			std::cerr << "Error: Could not set the character size." << std::endl ;
			exit(1) ;
		}
		//
		// Disable parity.
		//
		serial_port.SetParity( SerialStreamBuf::PARITY_NONE ) ;
		if ( ! serial_port.good() ) 
		{
			std::cerr << "Error: Could not disable the parity." << std::endl ;
			exit(1) ;
		}
		//
		// Set the number of stop bits.
		//
		serial_port.SetNumOfStopBits( 1 ) ;
		if ( ! serial_port.good() ) 
		{
			std::cerr << "Error: Could not set the number of stop bits."
				<< std::endl ;
			exit(1) ;
		}
		//
    		// Turn off all flow control.
    		//
    		serial_port.SetFlowControl( SerialStreamBuf::FLOW_CONTROL_NONE ) ;
    		if ( ! serial_port.good() ) 
    		{
        		std::cerr << "Error: Could not use hardware flow control."
				<< std::endl ;
			exit(1) ;
		}
		std::cout<<"K_serial::Serial Port Initialisation has completed."<<std::endl;
	
	
	}
	void writechar(char &_c){
		std::cout<<"K_serial::writechar("<<_c<<") to "<< SERIAL_PORT_DEVICE <<std::endl;
		serial_port.write( &_c, 1 );
		while(!wait_for_ack());
		//std::cout<<"writechar(char &_c) END"<<std::endl;
	}
	void writechar_noack(char &_c){
		std::cout<<"K_serial::writechar_noack("<<_c<<") to "<< SERIAL_PORT_DEVICE <<std::endl;
		serial_port.write( &_c, 1 );
		//std::cout<<"writechar(char &_c) END"<<std::endl;
	}
	void write_unsigned_char(unsigned char &_c){
		std::cout<<"K_serial::write_unsigned_char("<< std::setbase( 16 ) << (int)_c<< ") to "<< SERIAL_PORT_DEVICE <<std::endl;
		char temp_c =0x00;
		temp_c = temp_c | _c;
		serial_port.write( &temp_c, 1 );
	}
	void write_unsigned_int(unsigned int &_c){
		std::cout<<"K_serial::write_unsigned_int("<<_c<<") to "<< SERIAL_PORT_DEVICE <<std::endl;
		unsigned char temp_c =0x00;
		temp_c = _c >> 8; //write 3rd byte
		write_unsigned_char(temp_c);
		temp_c = _c; //write last byte
		write_unsigned_char(temp_c);
		while(!wait_for_ack());
		
	}
	void move_forward(unsigned int &_c){
		char temp_direction = 'F';
		writechar_noack(temp_direction);
		std::cout<<"K_serial::write_unsigned_int("<<_c<<") to "<< SERIAL_PORT_DEVICE <<std::endl;
		unsigned char temp_c =0x00;
		temp_c = _c >> 8; //write 3rd byte
		write_unsigned_char(temp_c);
		temp_c = _c; //write last byte
		write_unsigned_char(temp_c);
		while(!wait_for_ack());
	}
	void move_backward(unsigned int &_c){
		char temp_direction = 'B';
		writechar_noack(temp_direction);
		std::cout<<"K_serial::write_unsigned_int("<<_c<<") to "<< SERIAL_PORT_DEVICE <<std::endl;
		unsigned char temp_c =0x00;
		temp_c = _c >> 8; //write 3rd byte
		write_unsigned_char(temp_c);
		temp_c = _c; //write last byte
		write_unsigned_char(temp_c);
		while(!wait_for_ack());
	}
	int wait_for_ack(){
		volatile int read_flag =1;
		char read_byte='\0';
		int return_value=0;
		while( (read_flag == 1) && (serial_port.rdbuf()->in_avail() > 0)  ) 
		{
			serial_port.get(read_byte);
			//std::cerr << "read_byte_"<< read_byte << std::endl;
			if(read_byte== '1'){
				read_flag=0;
				return_value = 1;
			}else{
				return_value=0;
			}
		}
		//std::cout<<"return_value=="<<return_value<<std::endl;
		return (return_value);
	
	}
	int get_encoder_value(void){
		char c = 'A';
		serial_port.write( &c, 1 );
		volatile int read_flag =1;
		int count=0;
		char read_byte[7];
		char result_byte[6];
		while(read_flag)
		{
			while( (read_flag == 1) && (serial_port.rdbuf()->in_avail() > 0)  ) 
			{
				serial_port.get(read_byte[count]);
				std::cerr <<read_byte[count] << " ";
				if(read_byte[count] == '\n'){
					read_flag=0;
				}else{
					count++;
				}
				
				
			}
		}
		for(int i=0;i<count-1;i++)
		{
			result_byte[i]=read_byte[i+1];
		}
		std::cout<<"return("<< atoi(result_byte)<<")"<<std::endl;
		return(atoi(result_byte));
	}
	~K_serial(){
		//destructor;
		std::cout<<"K_serial::Destructing.."<<std::endl;
	}
	private:
	SerialStream serial_port;
	char* SERIAL_PORT_DEVICE;

};
