#ifndef CALIBRATION_H
#define CALIBRATION_H

#include "ui_bright.h"

#include <mimas/camera.h>
#include <mimas/image_draw.h>
#include <mimas/image_funcs.h>
#include <mimas/image_v4linput.h>
#include <mimas/mimasexception.h>

#include <QDialog>
#include <QMessageBox>

#include <algorithm>
#include <iomanip>
#include <iostream>

#include "points.h"
#include "picture.h"
#include "find_line.h"
#include "platform.h"
#include "config.h"

using namespace mimas;
using namespace camera_calibration;
using namespace std;

class bright : public QDialog, Ui::Dialog {

  Q_OBJECT

private:
  // laser calibration
  Picture *picts[2];
  bool is_picts[2];
  segment< point > *laser_lines[2];

  // platform calibration
  Picture *pict_platform;
  bool is_pict_platform;
  std::vector< pixel_f > points_platform[2];
  bool is_points_platform[2];

  // 3D reconstruction
  Picture *pict_3d;
  bool is_pict_3d;

  int size_x,
    size_y,
    p_size_x, 
    p_size_y,
    t_min,
    t_max,
    size_window,
    current,
    colour;
  
  double min_corr;

  string device;

public:
  bright(int size_x, int size_y, int p_size_x, int p_size_y, int min, int max, 
	 int size_window, double min_corr, char *device);

  Camera::Matrix getIntrinsics() throw (mimasexception);
  Camera::Vector getLaserNormal() throw (mimasexception);
  Camera::Vector getLaserPoint() throw (mimasexception);
  Camera::Vector getPlatformVector() throw (mimasexception);
  Camera::Vector getPlatformPoint() throw (mimasexception);
  double getStepDegrees() throw (mimasexception);

public slots:
  // laser calibration
  void takePicture1();
  void takePicture2();
  void clicCorners();
  void takeColours();
  void findCorners();
  void findLaser();
  void calibrateLaser();
  
  // platform calibration
  void takePicturePlatform();
  void clicCornersPlatform();
  void takeColoursPlatform();
  void findCornersLeft();
  void findCornersRight();
  void calibratePlatform();

  // 3D reconstruction
  void takePicture();
  void getPoints();
  void startScan();

};

#endif
