#include <QApplication>
#include <popt.h>
#include "bright.h"

int main( int argc, char *argv[] ) {

  // set variables to default
  int size_x = SIZE_X,
    size_y = SIZE_Y,
    min = T_MIN,
    max = T_MAX,
    p_size_x = PLATFORM_SIZE_X,
    p_size_y = PLATFORM_SIZE_Y,
    size_window = WINDOW;
  
  double min_corr = MIN_CORR;

  char *device = VIDEO_DEVICE;

  // popt options
  struct poptOption optionsTable[] = {

    { "device", 'd', POPT_ARG_STRING | POPT_ARGFLAG_SHOW_DEFAULT,
      &device, 0, "Specify video4linux device", "device-name" },

    { "width", 'w', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &size_x, 0, "Width of the chessboard (number of black squares)" ,
      "width" },

    { "height", 'h', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &size_y, 0, "Height of the chessboard (number of black squares)",
      "height"},

    { "pwidth", 'u', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &p_size_x, 0, "Width of the chessboard for the platform calibration" ,
      "width" },

    { "pheight", 'v', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &p_size_y, 0, "Height of the chessboard for the platform calibration",
      "height"},

    { "min", 'i', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &min, 0, "Minimum threshold for laser detection" ,
      "min" },

    { "max", 'a', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &max, 0, "Maximum threshold for laser detection",
      "max"},

    { "window", '\0', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &size_window, 0, "Size of the window for corner detection",
      "size"},

    { "correlation", '\0', POPT_ARG_DOUBLE | POPT_ARGFLAG_SHOW_DEFAULT,
      &min_corr, 0, "Specify minimum correlation to be calculated "
      "to assume that the corners are accurately found", "corr" },

    POPT_AUTOHELP

    POPT_TABLEEND 

  };

  poptContext optCon( poptGetContext( NULL, argc, (const char **)argv,
                                      optionsTable, 0 ) );

  try {

    // Parse command-line.
    char c;
    while ( ( c = poptGetNextOpt( optCon ) ) >= 0 );

    const char *strayParameter = poptGetArg( optCon );
    MMERROR( strayParameter == NULL, mimasexception, ,
             "Unexpected command-line argument \"" << strayParameter
             << "\"" );

    // Throw exception on error.
    MMERROR( c >= -1, mimasexception, ,
             poptBadOption( optCon, POPT_BADOPTION_NOALIAS ) << ": "
             << poptStrerror( c ) );

    QApplication app( argc, argv );

    bright h(size_x, size_y, p_size_x, p_size_y, min, max, size_window, min_corr, device);
    h.exec();

    return 0;

  } catch ( std::exception &e ) {

    cerr << e.what() << endl;
    poptPrintUsage( optCon, stderr, 0 );
    return 1;

  };
}
